module foregrounds

    implicit none
    private
    
    public :: foreground_params, utindex, InitForegroundData, cl_foreground, GetForegroundParamsFromArray, GetAlphaPrior, GetCirrusFactor, GetRadioAmpUncPrior, GetRadioAmpPrior, InitRadioAmpPrior
    type foreground_params
       !all czero's are D_3000 for that signal
        real czero_tsz
        real czero_ksz
        real czero_dg_po
        real czero_dg_cl
        real czero_cirrus(2) ! 1 = 150, 2 = 220
        real czero_rg

        real alpha_dg_po
        real sigma_dg_po
        real alpha_dg_cl
        real alpha_rg
        real sigma_rg
   
     end type foreground_params
     
     real, parameter :: pi = 3.1415926579
     
     ! Kinetic and thermal SZ effect templates
     real, dimension(2:10000) :: ksz_templ, tsz_templ, clust_dg_templ, clust2_dg_templ
     real, dimension(2:10000) :: cirrus_templ
     
     real DelAlphaPrior,CirrusFactorPrior,AddAlphaPoisson
     real UseRadioSigmaAsClusterAmp2,radio_amp,radio_unc
   contains


    !
    ! Returns in cl_arr the matrix
    !       (TT, TE, TB)
    !       (ET, EE, EB)
    !       (BT, BE, BB)
    !
    ! corresponding to the point source cross/auto power spectrums
    ! at the given frequency and at one single l-value
    !
    ! the logical array which turns on/off different componenets. the order is
    !   (dutsy clustered, dusty poisson, radio poisson, ksz, tsz)
    !
    ! the array bytype returns the TT components in that same order
    !
    function cl_foreground(params,l,ifr,jfr,nfr,eff_fr,norm_fr)
        type(foreground_params) :: params
        integer :: i,l,ifr,jfr,nfr
        real, dimension(5,nfr) :: eff_fr
        real, dimension(5) :: norm_fr
        real :: cl_foreground
        real :: fri,frj,norm,fr0,frqdep
        real :: cl_dg_po, cl_dg_cl, cl_rg_po, cl_ksz, cl_tsz, cl_cirrus
        real :: cl_dg, cl_rg
        real :: bytype(5)
        real, parameter :: d3000 = 3000*3001/(2*pi)
        real :: czero_cirrus
        real :: effalpha_cl
        real :: effsigma_rg, effclusamp2

        ! dusty clustered
        fr0 = norm_fr(1)
        fri = eff_fr(1,ifr)
        frj = eff_fr(1,jfr)

        ! The main power law frequency dependence
        effalpha_cl = params%alpha_dg_cl + AddAlphaPoisson*params%alpha_dg_po
        frqdep = (fri/fr0*frj/fr0)**effalpha_cl / dBdT(fri,fr0) / dBdT(frj,fr0)

        !Sep 15, 2010 - took out the ell-dependence.
        ! The smaller l dependent dependence
!        frqdep = frqdep * (l/3000.)**(-.0000184825*((fri-fr0)+(frj-fr0)))
        norm = 1./(d3000*clust_dg_templ(3000))

        cl_dg_cl = clust_dg_templ(l)*params%czero_dg_cl*norm*frqdep

        if (UseRadioSigmaAsClusterAmp2 .EQ. 1) then
           effsigma_rg = 0
           effclusamp2 = params%sigma_rg
           norm = 1./(d3000*clust2_dg_templ(3000))
           cl_dg_cl = cl_dg_cl + clust2_dg_templ(l)*effclusamp2*norm*frqdep
        else
           effsigma_rg = params%sigma_rg
           effclusamp2 = 0
        end if


        ! dusty poisson
        fr0 = norm_fr(2)
        fri = eff_fr(2,ifr)
        frj = eff_fr(2,jfr)
        norm = 1./d3000
        cl_dg_po = norm*params%czero_dg_po/dBdT(fri,fr0)/dBdT(frj,fr0)*&
             (fri/fr0*frj/fr0)**(params%alpha_dg_po + &
             alog(fri/fr0*frj/fr0)/2 * params%sigma_dg_po**2)
!             *(1+params%sigma_dg_po**2*alog(fri/fr0)*alog(frj/fr0))
        
        ! radio poisson
        fr0 = norm_fr(3)
        fri = eff_fr(3,ifr)
        frj = eff_fr(3,jfr)
        norm = 1./d3000
        cl_rg_po = norm*params%czero_rg/dBdT(fri,fr0)/dBdT(frj,fr0)*&
             (fri/fr0*frj/fr0)**(params%alpha_rg + &
             alog(fri/fr0*frj/fr0)/2 * effsigma_rg**2)
        !(fri/fr0*frj/fr0)**params%alpha_rg&
         !    *(1+params%sigma_rg**2*alog(fri/fr0)*alog(frj/fr0))/dBdT(fri,fr0)/dBdT(frj,fr0)
        
        ! ksz
        fr0 = norm_fr(4)
        fri = eff_fr(4,ifr)
        frj = eff_fr(4,jfr)
        norm = 1. / (d3000*ksz_templ(3000))
        cl_ksz = params%czero_ksz * norm * ksz_templ(l)
        
        ! tsz
        fr0 = norm_fr(5)
        fri = eff_fr(5,ifr)
        frj = eff_fr(5,jfr)
        norm = 1. /(d3000*tsz_templ(3000))
        cl_tsz = params%czero_tsz * norm * tsz_templ(l) * tszFreqDep(fri,fr0) * tszFreqDep(frj,fr0)

        norm = 1. /(d3000*cirrus_templ(3000))
        czero_cirrus = sqrt(params%czero_cirrus(ifr)*params%czero_cirrus(jfr))
        cl_cirrus = czero_cirrus * norm * cirrus_templ(l)

        cl_dg = cl_dg_po + cl_dg_cl
        cl_rg = cl_rg_po

        cl_foreground = cl_dg+cl_rg+cl_tsz+cl_ksz + cl_cirrus

    end function

    !
    ! nu,nu0 in GHz
    !
    ! dBdT is proportional to derivative of planck function
    ! but is normalized so its equal to 1 at nu0
    !
    function dBdT(nu,nu0)

        real x, x0, dBdT, dBdT0, nu, nu0

        x0 = nu0/56.78
        dBdT0 = x0**4 * exp(x0) / (exp(x0)-1)**2

        x = nu/56.78
        dBdT = x**4 * exp(x) / (exp(x)-1)**2 / dbdT0

    end function

    !
    ! nu, nu0 in GHz
    ! Gives the tsz frequency dependence normalized so its 1 at nu0
    !
    function tszFreqDep(nu,nu0)

        real :: tszFreqDep, tszFreqDep0, nu, nu0, x, x0

        x = nu / 56.78
        x0 = nu0 / 56.78

        tszFreqDep0 = x0*(exp(x0)+1)/(exp(x0)-1) - 4
        tszFreqDep = x*(exp(x)+1)/(exp(x)-1) - 4

        tszFreqDep = tszFreqDep/tszFreqDep0

    end function


    !
    ! Loads the clustered, ksz, and tsz templates which are expected to be in
    ! foreground_folder with names cluster_*.dat, ksz.dat, and tsz.dat
    !
    subroutine InitForegroundData(fClustered,fClustered2,fKSZ,fTSZ,del_alpha,cirrus_factor,relative_alpha_cluster)
      integer :: l,dum
      character(len=120) :: file
      character*(*) :: fClustered, fKSZ, fTSZ,fClustered2
      real :: del_alpha,cirrus_factor
      logical :: relative_alpha_cluster
      
      ! TSZ template
      Open(10,file=fClustered)!foreground_folder//'clustered_150.dat')
      Do l = 2,10000
         Read (10,*) dum, clust_dg_templ(l)
      End Do
      Close(10)
      
      ! KSZ template
      Open(10,file=fKSZ)!foreground_folder//'ksz.dat')
      Do l = 2,10000
         Read (10,*) dum, ksz_templ(l)
      End Do
      Close(10)
      
      ! TSZ template
      Open(10,file=fTSZ)!foreground_folder//'tsz.dat')
      Do l = 100,10000
         Read (10,*) dum, tsz_templ(l)
      End Do
      Close(10)
      !prior on separation between clsutered and Poisson terms
      ! ignored if <= 0
      DelAlphaPrior = del_alpha
      
      CirrusFactorPrior = cirrus_factor
      Do l = 200,10000
         cirrus_templ(l) = (l/3000.) ** (-3.5)
      enddo
      
      AddAlphaPoisson = 0
      if (relative_alpha_cluster) then
         AddAlphaPoisson = 1
      end if

      if ( fClustered2/='') then
         Open(10,file=fClustered2)!foreground_folder//'clustered_150.dat')
         Do l = 2,10000
            Read (10,*) dum, clust2_dg_templ(l)
         End Do
         Close(10)
         UseRadioSigmaAsClusterAmp2 = 1
      else
         UseRadioSigmaAsClusterAmp2 = 0
         clust2_dg_templ(:)=0
      end if

    end subroutine InitForegroundData

    subroutine InitRadioAmpPrior(iradio_amp,iradio_unc)
      real::iradio_amp, iradio_unc
      radio_amp=iradio_amp
      radio_unc=iradio_unc
    end subroutine InitRadioAmpPrior
    
    function GetRadioAmpPrior()
      real :: GetRadioAmpPrior
      GetRadioAmpPrior=radio_amp
    end function GetRadioAmpPrior

    function GetRadioAmpUncPrior()
      real :: GetRadioAmpUncPrior
      GetRadioAmpUncPrior=radio_unc
    end function GetRadioAmpUncPrior
    
    !
    ! returns alpha prior on Poisson-cluster index
    !
    function GetAlphaPrior()
      real :: GetAlphaPrior
      GetAlphaPrior = DelAlphaPrior
    end function

    !
    ! returns cirrus prior pre-factor
    ! <0 is no prior.
    function GetCirrusFactor()
      real :: GetCirrusFactor
      GetCirrusFactor = CirrusFactorPrior
    end function
    

    !
    ! Index of the (i,j) entries of the upper triangular part of an n-by-n matrix
    !
    recursive function utindex(i,j,n)

        integer :: i, j, n, utindex

        if (i <= j) then
            utindex = (i-1)*n+j - i*(i-1)/2
        else
            utindex = utindex(j,i,n)
        end if

    end function

    ! Read parameters from an array of reals ordered in the same order as the type
    ! Basically just cast the array as the type and the parameters line up
    function GetForegroundParamsFromArray(array)

    	real, dimension(:) :: array
    	type(foreground_params) :: GetForegroundParamsFromArray

!        write(*,*)array
    	GetForegroundParamsFromArray = transfer(array,GetForegroundParamsFromArray)

   	end function

end module
